﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.FSharp.Collections;

namespace PlusOne.Collections
{
	public class ImmutableMap<TKey, TValue>
		where TKey : IComparable
	{
		private readonly FSharpMap<TKey, TValue> innerMap;

		private ImmutableMap(FSharpMap<TKey, TValue> innerMap)
		{
			this.innerMap = innerMap;
		}

		public ImmutableMap(IEnumerable<Tuple<TKey, TValue>> pairs)
		{
			innerMap = new FSharpMap<TKey, TValue>(pairs);
		}

		public ImmutableMap()
			: this(MapModule.Empty<TKey, TValue>())
		{
		}

		public TValue this[TKey key]
		{
			get
			{
				return innerMap[key];
			}
		}

		public IEnumerable<KeyValuePair<TKey, TValue>> Pairs
		{
			get
			{
				return innerMap;
			}
		}

		public IEnumerable<TValue> Values
		{
			get
			{
				return innerMap.Select(x => x.Value);
			}
		}

		public ImmutableMap<TKey, TValue> Add(TKey key, TValue value)
		{
			return innerMap.Add(key, value);
		}

		public static implicit operator ImmutableMap<TKey, TValue>(FSharpMap<TKey, TValue> inner)
		{
			return new ImmutableMap<TKey, TValue>(inner);
		}
	}
}